module SMSCentre
  class Configuration
    # @return [Boolean] use HTTPS by default.
    #   Defaults to false.
    attr_accessor :ssl

    # @return [String] SMS Centre gateway API host.
    #   Defaults to "smsc.ru".
    attr_accessor :host

    # @return [Fixnum] SMS Centre gateway API port.
    #   Defaults to 80 or 443, depends on 'ssl' option.
    def port
      @port ||= ssl ? 443 : 80
    end

    attr_writer :port

    # @return [String] Default sender name.
    #   Defaults to nil.
    attr_accessor :sender

    # @return [Fixnum] Default timeout for single API call.
    #   Defaults to 0 (no timeout).
    attr_accessor :timeout

    # @return [Logger] Logger instance, should be compatible with ruby logger.
    #   Defaults to Logger.new(STDOUT).
    attr_accessor :logger

    # @return [Fixnum] Maximum phones count for single API call.
    #   Defaults to 10.
    attr_accessor :max_phones_per_request

    def initialize
      @ssl                    = true
      @host                   = "smsc.ru"
      @timeout                = 0
      @max_phones_per_request = 10

      @logger ||= Logger.new(STDOUT)
      @logger.level = Logger::DEBUG
    end
  end
end
